package read;
/*****************************************************************************/
/* PipeReader.java                                                           */
/* Driver/test program to test PRS read pipe support from java.              */
/* Author: John Foley                                                        */
/* Date:   10/30/02                                                          */
/* Copyright 2002, QVS Software, Inc. All rights reserved.                   */
/*****************************************************************************/

import java.io.IOException;
import java.util.Vector;
import java.io.*;

import qvs.*;
import qvs.RPAMEventListener;
import qvs.RPAMEvents.*;
import qvs.RPAMDevices.*;

import qvs.RPAMApp;
import qvs.RPAMTerminal;
import qvs.RPAMDevices.POSDisplay;

/*****************************************************************************/
/* PipeReader Class                                                          */
/*****************************************************************************/
public class PipeReader implements qvs.RPAMGlobals
   {
   public static void ParseArgs(String[] args)
      {
                                       /* a not too sophisticated cmd line   */
                                       /* arg parser.  Options supported:    */
                                       /* -t <TermNum> and -s <StoreNum>     */
      int i;
      for (i = 0; i < args.length; i++)
         {
         if (args[i].compareTo("-t") == 0)
            {
            i++;
            TermNum = (new Integer(args[i])).intValue();
            System.out.println("Term overridden to " + TermNum);
            }
         else if (args[i].compareTo("-u") == 0)
            {
            i++;
            UnitID = (new Integer(args[i])).intValue();
            System.out.println("UnitID overridden to " + TermNum);
            }
         else if (args[i].compareTo("-s") == 0)
            {
            i++;
            StoreNum = (new Integer(args[i])).intValue();
            System.out.println("Store overridden to " + StoreNum);
            }
         }
      }

   public synchronized void Wakeup()
      {
      notifyAll();                     /* wake up all threads waiting on our */
                                       /* monitor                            */
      }

   public PipeReader()                 /* default constructor                */
      {                                /* create the objects we will use     */
      Handler = new EventHandler();    /* handles terminal events            */

      Term = new RPAMTerminal();       /* the Terminal object                */
      App = new RPAMApp();             /* the RPAM app object for trace etc. */
      BufferedKeys = new Vector();     /* queue of keys to send to the .286  */

      rdr = new KeyboardReader();      /* A thread to read the keyboard      */

      TermNum = 1;                     /* default to terminal number 1       */
      StoreNum = 9;                    /* default to store number 168        */
      FirstTime = 1;
      UnitID = 1;
      }
                                       /* local class to handle events fired */
                                       /* from the terminal object           */
   public class EventHandler implements qvs.RPAMEventListener
      {
      private int PrintCount = 0;

      public void IOProcWait(IOProcWaitEvent evt) { }
      public void Tone(ToneEvent evt) { }
      public void Offline(OfflineEvent evt) { };
      public void ConnectTimeout(ConnectTimeoutEvent evt) { };
      public void Display(DisplayEvent evt) { };
      public void Print(PrintEvent evt) { }
      public void PrintEx(PrintExEvent evt) { }
      public void ActivateMICR(ActivateMICREvent evt) { }
      public void Lock(LockEvent evt) { };
      public void Unlock(UnlockEvent evt) { };
      public void ScannerUnlock(ScannerUnlockEvent evt) { };
      public void ScannerLock(ScannerLockEvent evt) { };
      public void Online(qvs.RPAMEvents.OnlineEvent evt) { }
      public void DisplayEx(qvs.RPAMEvents.DisplayExEvent evt) { }
      public void WaitLight(WaitLightEvent evt) { };
      public void MessageLight(MessageLightEvent evt) { };
      public void OfflineLight(OfflineLightEvent evt)
         {
         if (evt.NewValue == 0)
            {
            System.out.println("Online to the controller");
            }
         };

      public void FieldLevelInput2(FieldLevelInput2Event evt) { };
      public void FieldLevelInput(FieldLevelInputEvent evt) { };
      public void MsrLock(MsrLockEvent evt) { }
      public void MsrUnlock(MsrUnlockEvent evt) { }
      public void ScaleRead(ScaleReadEvent evt) { }
      public void UserData(UserDataEvent evt) { };
      public void RunningTotalData(RunningTotalDataEvent evt) {};
      public void TotalsWrite(TotalsWriteEvent evt) {};
      public void TotalsRead(TotalsReadEvent evt) {};
      public void TermNumberChanged(TermNumberChangedEvent evt) {};
      public void OfflineTotalsData(OfflineTotalsDataEvent evt) {};
      public void PrintEject(PrintEjectEvent evt) {};
      public void ReceiptCut(ReceiptCutEvent evt) {};
      public void TermIPLControl(TermIPLControlEvent evt) {};
      public void TermReload(TermReloadEvent evt) {};
      public void ActivateMarqueeDisplay(ActivateMarqueeDisplayEvent evt) {};
      public void DeactivateMarqueeDisplay(DeactivateMarqueeDisplayEvent evt) {};
      public void LoadMarqueeDisplay(LoadMarqueeDisplayEvent evt) {};
      public void DrawerOpen(DrawerOpenEvent evt) {};
      }
                                       /* local class to run a thread to     */
                                       /* read the keyboard                  */
   public class KeyboardReader extends Thread
      {
      char[] InKeys = new char[100];
      int CharsRead = 0;
      int x;
      InputStreamReader isr = new InputStreamReader(System.in);

      char[] KeyMap =                  /* table to translate ASCII key codes */
                                       /* to SA function codes               */
         {

         /*
         Ascii Key      SA FunctionCode
         */

         '/',           (char)0x4e,    /* /               */
         '0',           (char)0x30,    /* 0               */
         '1',           (char)0x31,    /* 1               */
         '2',           (char)0x32,    /* 2               */
         '3',           (char)0x33,    /* 3               */
         '4',           (char)0x34,    /* 4               */
         '5',           (char)0x35,    /* 5               */
         '6',           (char)0x36,    /* 6               */
         '7',           (char)0x37,    /* 7               */
         '8',           (char)0x38,    /* 8               */
         '9',           (char)0x39,    /* 9               */
         's',           (char)0x3d,    /* Signon  S       */
         'c',           (char)0x49,    /* Clear   E       */
         'e',           (char)0x50,    /* Enter   R       */
         't',           (char)0x51,    /* Total   T       */
         'h',           (char)0x5b,    /* Cash    H       */
         'w',           (char)0x48,    /* Weight          */
         (char)0x00,    (char)0x00     /* End of data     */
         };

//char[] KeyMap =                  /* table to translate ASCII key codes */
//                                     /* to GSA function codes              */
//       {
//
//       /*
//       Ascii Key      GSA FunctionCode
//       */
//
//       '/',           78,    /* /               */
//       '0',           48,    /* 0               */
//       '1',           49,    /* 1               */
//       '2',           50,    /* 2               */
//       '3',           51,    /* 3               */
//       '4',           52,    /* 4               */
//       '5',           53,    /* 5               */
//       '6',           54,    /* 6               */
//       '7',           55,    /* 7               */
//       '8',           56,    /* 8               */
//       '9',           57,    /* 9               */
//       's',           76,    /* Signon          */
//       'c',           73,    /* Clear           */
//       'e',           95,    /* Enter           */
//       't',           75,    /* Total           */
//       'h',           69,    /* Cash            */
//       'k',          102,    /* Credit          */
//       (char)0x00,    (char)0x00     /* End of data     */
//       };

                                       /* routine to translate keystrokes to */
                                       /* function codes                     */

      public int GetFCodeFromKey(int InKey)
         {
         int RetFCode = 0;
         int idx;
         for(idx=0;(KeyMap[idx] != 0);idx+=2)
            {
            if (KeyMap[idx] == InKey)
               {
               RetFCode = KeyMap[idx+1];
               break;
               }
            }
         return RetFCode;
         }

      public void run()
         {
         int FCode;
         try
            {
            for(;;)                    /* loop forever                       */
               {                       /* block waiting for some keystrokes  */
               CharsRead = isr.read(InKeys,0,1);
               for (x=0;x<CharsRead;x++)
                  {
                  if (InKeys[x] == 'r')
                     {                 /* write testdata to the pipe         */
                     DoPRSRead();
                     }
                  else
                     {
                     FCode = GetFCodeFromKey((int)InKeys[x]);
                     if (FCode > 0)
                        {
                        Term.getKeyboard().SendKeyboardData(FCode);
                        }
                     }
                  }
               }
            }
         catch(IOException ex)
            {
            System.err.println("IOException: " +  ex.getMessage());
            }
         }
      }

   EventHandler Handler;               /* an instance of the event handler   */
                                       /* interface                          */
   RPAMTerminal Term;                  /* a Terminal object to use           */
   RPAMApp App;                        /* an App objec to use                */
   Vector BufferedKeys;                /* queued keys to send                */
   KeyboardReader rdr;                 /* input reader thread                */
   static int TermNum;                 /* terminal number we'll connect as   */
   static int StoreNum;                /* our store number                   */
   static int UnitID;                  /* our unit id                        */

   static int FirstTime;
   POSPrsReadPipe rPipe;

                                       /* app main line code                 */
   public static void main(String args[])
      {
      char InChar;
      int Avail;


      PipeReader Dvr = new PipeReader(); /* get an instance of the app class   */
                                       /* created                            */

      Dvr.ParseArgs(args);             /* get our cmd line arguments         */

                                       /* add our event handler to the list  */
                                       /* of listener for RPAM events from   */
                                       /* the terminal object                */
      Dvr.Term.addRPAMEventListener(Dvr.Handler);

      Dvr.App.setTracing(1);           /* turn RPAM tracing on               */

      Dvr.App.setTermNumAssignMode(1); /* we're going to assing term nums    */

      Dvr.Term.setUnitID(UnitID);      /* set our unit id                    */

      Dvr.Term.setTermNum(TermNum);    /* set our terminal number            */

      Dvr.App.setStoreNum(StoreNum);   /* TESTING purposes only.  Should     */
                                       /* never set StoreNum. Note: for now  */
                                       /* you must do this after you set     */
                                       /* unit id for the terminal.          */

                                       /* tell the .286 we have a mod 3/4    */
                                       /* printer attached                   */
      Dvr.Term.getPrinter().setDeviceID(rpaMod34prt);

      Dvr.Term.Connect();              /* get the TC session rolling...      */
      System.out.println("Waiting for online to controller");

      Dvr.rdr.start();                 /* get the keyboard thread started    */

      for (;;)                         /* loop forever                       */
         {
         synchronized(Dvr)
            {
            try
               {
               Dvr.wait();             /* wait for something to happen       */
               }
            catch(InterruptedException ex)
               {
               System.err.println("InterruptedException: " +  ex.getMessage());
               }
            }
         }
      }

   public String ByteArrToString(byte[] arr, int start, int len)
      {
      String RetVal = "";
      int i;
      for (i=start;i<start+len;i++)
         {
         RetVal = RetVal + (char)arr[i];
         }
      return RetVal;
      }

   public byte[] StringToByteArr(String str)
      {
      int Len = str.length();
      byte[] RetVal = new byte[Len];
      int i;
      for (i=0;i<Len;i++)
         {
         RetVal[i] = (byte)(str.charAt(i));
         }
      return RetVal;

      }

   public void DoPRSRead()
      {
      int hr = S_OK;
      int EC4690;
      byte [] buf = new byte[0x200];
      int x;

      System.out.println("Doing PRS read");


      if (FirstTime == 1)
         {
                                       /* first time through, create the     */
                                       /* 'J' pipe                           */
         rPipe = new POSPrsReadPipe();
         rPipe.setBufSize(240);
         rPipe.setPipeID((byte)'J');
         rPipe.setUnitID(UnitID);

         hr = rPipe.Create();
         EC4690 = rPipe.getErrorCode4690();
         if (EC4690 == 0)
            {
            System.out.println("'J' Pipe created");
            FirstTime = 0;
            }
         else
            {
            System.out.println("4690 error creating 'J' pipe, rc " + EC4690);
            hr = 0xFFFFFFFF;
            FirstTime = 1;
            }
         }

                                          /* if everything looks ok, issue a    */
                                          /* read to the pipe, timeout = 5secs  */
      if (hr >= 0)
         {
                                          /* the pipe writer sample first writes*/
                                          /* a 1 byte length to the pipe and    */
                                          /* then the data so read the length   */
                                          /* byte out first                     */
         hr = rPipe.Read(buf, 1, 5000);
         EC4690 = rPipe.getErrorCode4690();
         if (EC4690 == 0)
            {
            hr = (int)buf[0];
            System.out.println("Length to read from pipe is " + hr + " bytes");
           }
         else
            {
            if (EC4690 == errReadTimedOut)
               {
               System.out.println("Read for length timed out\n");
               }
            else
               {
               System.out.println("4690 error reading length from 'J' pipe, rc " + EC4690);
               }
                                       /* prevent the code below from running*/
            hr = -1;
            }

         if (hr > 0)
            {
            hr = rPipe.Read(buf, hr, 5000);
            EC4690 = rPipe.getErrorCode4690();
            if (EC4690 == 0)
               {
               System.out.println("Data from pipe");
               hexlog(buf, hr, "PipeData");
               }
            else
               {
               if (EC4690 == errReadTimedOut)
                  {
                  System.out.println("Read for data timed out");
                  }
               else
                  {
                  System.out.println("4690 error reading data from 'J' pipe, rc" +  EC4690);
                  }
               }

            }
         }
      }

   public static void hexlog(byte [] buf, int buflen, String label)
      {
      String OutStr = "";
      int x, y=0;
      int hdrlen = label.length() + 8;
      String temp;

      for (x=0;x<buflen;x++)
         {
         if ((x % 16) == 0)
            {
            if (OutStr.length() != 0)
               {
               System.out.println(OutStr);
               }

            OutStr = " " + label + " " + FormatHexString(x, 4) +
                ":                                         *                *";
            y = 0;
            }

         temp = FormatHexString((int)buf[x], 2);
         OutStr = OutStr.substring(0, hdrlen + (y*2) + (y/2))
                         + temp + OutStr.substring(hdrlen + (y*2) + (y/2) + 2);
         if ((buf[x] >= 32) && (buf[x] <= 126))
            {
            OutStr = OutStr.substring(0, hdrlen + 40 + y +1) + (char)buf[x] +
                                         OutStr.substring(hdrlen + 40 + y + 2);
            }

         y++;
         }

      System.out.println(OutStr);
      }
   public static String FormatHexString(int Number, int MaxLen)
      {
      String    OutStr;

      OutStr = Integer.toHexString(Number);

      if (OutStr.length() > MaxLen)
         {
         OutStr = OutStr.substring(OutStr.length()-1-MaxLen,
         OutStr.length()-1);
         }
      else if (OutStr.length() < MaxLen)
         {
         while (OutStr.length() < MaxLen)
            {
            OutStr = "0" + OutStr;
            }
         }

      return OutStr;
      }
   }


